/*
 *  trw_s.cpp
 *  dtbca_public_release
 *
 *  Created by Dhruv Batra on 1/19/11.
 *  Copyright 2011. All rights reserved.
 *
 */

#include "trw_s.h"
#include "MRFEnergy.h"
#include "constants_declaration.h"

void TRWSwrapper(int nnodes, int nedges, int *cardinalities, int* edgelist, double* D, double* V)
{
    
    ////////////////////////////////////////////////////////////////////////////////
    // Construct TRW-S Graph (Based on type of problem)
#ifdef TYPE_POTTS
	MRFEnergy<TypePotts>* mrf;
	MRFEnergy<TypePotts>::NodeId* nodes;
	MRFEnergy<TypePotts>::Options options;
	vector<TypePotts::REAL> enhist, lbhist;
    
    // Potts-Model implies that all nodes have same cardinality. Check that.
    int nlabels = cardinalities[0];
    for (int i=1; i!=nnodes; ++i)
        CHECK((nlabels==cardinalities[i]), "For Potts Model all nodes must have equal cardinalities");
    
    mrf = new MRFEnergy<TypePotts>(TypePotts::GlobalSize(nlabels));
    nodes = new MRFEnergy<TypePotts>::NodeId[nnodes];
    
    // Add node potentials
    for (int i=0; i!=nnodes; ++i) 
    {
        CHECK_NOT_NULL(D+i*nlabels, "Adding NULL unary potential");
		nodes[i] = mrf->AddNode(TypePotts::LocalSize(), TypePotts::NodeData(D+i*nlabels));
    }
    // Add edge potentials
    int ni, nj;
    for (int e=0; e!=nedges; ++e) 
    {
        ni = edgelist[2*e];
        nj = edgelist[2*e+1];
        
        mrf->AddEdge(nodes[ni], nodes[nj], TypePotts::EdgeData(V[e]));
    }
    
	// Function below is optional - it may help if, for example, nodes are added in a random order
	// mrf->SetAutomaticOrdering();
    
#else // General Problem
 	MRFEnergy<TypeGeneral>* mrf;
	MRFEnergy<TypeGeneral>::NodeId* nodes;
	MRFEnergy<TypeGeneral>::Options options;
	vector<TypeGeneral::REAL> enhist, lbhist;
    
    mrf = new MRFEnergy<TypeGeneral>(TypeGeneral::GlobalSize());
	nodes = new MRFEnergy<TypeGeneral>::NodeId[nnodes];
    
    // Add node potentials
    int cardsum = 0;
	for (int i=0; i!=nnodes; ++i) 
    {
        CHECK_NOT_NULL(D+cardsum, "Adding NULL unary potential");
        
        nodes[i] = mrf->AddNode(TypeGeneral::LocalSize(cardinalities[i]), 
                                TypeGeneral::NodeData(D + cardsum));
        cardsum += cardinalities[i];
	}
    
    // Add edge potentials
	int ni, nj;
	cardsum = 0;
	for (int e = 0; e < nedges; ++e) {
        CHECK_NOT_NULL(V+cardsum, "Adding NULL pairwise potential");
        
        ni = edgelist[2*e];
        nj = edgelist[2*e+1];
        mrf->AddEdge(nodes[ni], nodes[nj], TypeGeneral::EdgeData(TypeGeneral::GENERAL, V + cardsum));
        cardsum += cardinalities[ni] * cardinalities[nj];
	}
    
	//mrf->SetAutomaticOrdering(options);
#endif
    
    ////////////////////////////////////////////////////////////////////////////////
    // Parameters
    options.m_iterMax = MAXITER; // maximum number of iterations
	options.m_printIter = 0;
	options.m_printMinIter = 0;
	options.m_eps = DUAL_IMPROV_THRES;
	options.m_maxtime = MAXTIME;
    options.m_verbose_level = 1;
    
#ifndef TYPE_POTTS
    mrf->SetAutomaticOrdering(options);
#endif
    
    ////////////////////////////////////////////////////////////////////////////////
    // Run TRW-S algorithm 
    vector<double> timehist, disthist;
	mrf->Finalize_TRW_S(options, lbhist, enhist, timehist, disthist);
    
	if (LOAD_MESSAGES) {
        ifstream serialized_msgs_in;
        string serialized_filename = name + "_trws_serialized_msgs.dat";
        serialized_msgs_in.open(serialized_filename.c_str());
        mrf->LoadMessages(serialized_msgs_in);
        serialized_msgs_in.close();	  
	}
    
	mrf->Minimize_TRW_S(options, lbhist, enhist, timehist, disthist);
    
    ////////////////////////////////////////////////////////////////////////////////
	// Save data to files
	ostringstream o; o << "_alg_t" << -1 << ".txt";
    
    WriteToFile(outputpath+"enhist"+o.str(), enhist);
    WriteToFile(outputpath+"lbhist"+o.str(), lbhist);
    WriteToFile(outputpath+"timehist"+o.str(), timehist);
    WriteToFile(outputpath+"disthist"+o.str(), disthist);
    
    // Get Primal Solution
    vector<int> soln(nnodes,0);
    for (int n=0; n!=nnodes; ++n)
        soln[n] = mrf->GetSolution(nodes[n]);
    WriteToFile(outputpath+"primalsol"+o.str(), soln);
    
	if (SAVE_MESSAGES) 
    {
        ofstream serialized_msg_file;
        string serialized_filename = name + "_trws_serialized_msgs.dat";
        serialized_msg_file.open(serialized_filename.c_str());
        mrf->SerializeMessages(serialized_msg_file);
        serialized_msg_file.close();
	}
    
    ////////////////////////////////////////////////////////////////////////////////
	// Destruction and Done
	delete nodes;
	delete mrf;
    
}

